""" Builder's class of the gpao """

import json
from gpao.project import Project


def handler(obj):
    """ handler"""
    dictionary = obj.__dict__.copy()
    del dictionary["internal_id"]

    # Remove empty key (deps, tags) and store in a new dictionary tmp
    dictionary = {k: v for k, v in dictionary.items() if v}

    return dictionary


class Builder(object):
    """ Builder of GPAO's json project """

    def __init__(self, projects=None):
        """constructeur"""

        self.projects = []

        if projects is not None:
            self.projects.extend(projects)

    def add_project(self, project):
        """ Add project """
        self.projects.append(project)

    def save_as_json(self, file_json):
        """ Write Json File """

        json_gpao = {"projects": []}
        for project in self.projects:
            Project.reorganize_job_dependencies(project)
            json_gpao["projects"].append(project)
        with open(file_json, "w") as fjson:
            json.dump(
                json_gpao,
                fjson,
                default=handler,
                ensure_ascii=False,
                indent=4
                )



""" Porject's class of the GPAO"""

import json

from gpao.job import Job


def handler(obj):
    """ handler """
    if isinstance(obj, Job):
        dictionary = obj.__dict__.copy()
        del dictionary["internal_id"]

        dictionary = {k: v for k, v in dictionary.items() if v}
        return dictionary
    return None


class Project(object):

    """ Project classe """

    cpt_id = 0

    def __init__(self, name, jobs=None, deps=None):
        self.name = name

        self.jobs = []
        self.jobs.extend(jobs)

        self.deps = []
        if deps is not None:
            if isinstance(deps, Project):
                self.deps.append({"id": deps.get_internal_id()})
            elif isinstance(deps, list):
                for project in deps:
                    self.deps.append({"id": project.get_internal_id()})

        self.internal_id = Project.cpt_id
        Project.cpt_id += 1

    def get_internal_id(self):
        """ Get internal id """
        return self.internal_id

    def add_dependency(self, dep):
        """ Add a dependencie to this job"""
        if isinstance(dep, Project):
            self.deps.append({"id": dep.get_internal_id()})
        else:
            self.deps.append(dep)

    def add_job(self, job):
        """ Add job into project """
        self.jobs.append(job)

    def get_name(self):
        """ Get name of the project """
        return self.name

    @staticmethod
    def reorganize_job_dependencies(project):
        """ Reorganize job's id in deps list """
        for job in project.jobs:
            if job.deps:
                for dep in job.deps:
                    job_id = dep['id']
                    dep['id'] = Project.find_job_index(project, job_id)

    @staticmethod
    def find_job_index(project, job_id):
        """ find job's index in project and return this position """
        cpt = 0
        for job in project.jobs:
            if job.get_internal_id() == job_id:
                return cpt
            cpt += 1

        return -1

    def to_json(self):
        """ Convert to json """
        # del internal id from project
        dictionary = self.__dict__.copy()
        del dictionary["internal_id"]

        dictionary = {k: v for k, v in dictionary.items() if v}

        Project.reorganize_job_dependencies(self)

        return json.dumps(dictionary, default=handler, indent=4)



""" Job's class of the gpao """

import json


class Job(object):
    """ Job's Class """

    cpt_id = 0

    def __init__(self, name, command, deps=None, tags=None):
        """ constructor"""

        self.name = name
        self.command = command

        self.deps = []

        if deps is not None:
            if isinstance(deps, Job):
                self.deps.append({"id": deps.get_internal_id()})
            elif isinstance(deps, list):
                for job in deps:
                    self.deps.append({"id": job.get_internal_id()})

        self.tags = []
        if tags is not None:
            self.tags.extend(tags)

        self.internal_id = Job.cpt_id
        Job.cpt_id += 1

    def get_internal_id(self):
        """ Get internal id """
        return self.internal_id

    def add_dependency(self, dep):
        """ Add a dependencie to this job"""
        if isinstance(dep, Job):
            self.deps.append({"id": dep.get_internal_id()})
        else:
            self.deps.append(dep)

    def to_json(self):
        """ convert to json string"""
        # del internal id from job
        dictionary = self.__dict__.copy()
        del dictionary["internal_id"]

        dictionary = {k: v for k, v in dictionary.items() if v}

        return json.dumps(dictionary, indent=4)



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="ign-gpao-project-builder",
    version="0.4.1",
    description="Write a json's GPAO file",
    long_description_content_type="text/markdown",
    long_description=README,
    url="https://github.com/arnaudbirk/ign-gpao-project-builder.git",
    author="Arnaud Birk",
    author_email="arnaud.birk@ign.fr",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    packages=["gpao"],
    include_package_data=True,
)
